use std::fmt::Display;

pub enum Generation {
    One = 1,
    Two = 2,
    Three = 3,
    Four = 4,
    Five = 5,
    Six = 6,
    Seven = 7,
    Eight = 8,
    Nine = 9,

    /// the Let's Go gamess
    Gen7b,
    /// Legends Arceus
    Gen8a,
    /// BDSP
    Gen8b,
}

pub enum Nature {
    Hardy = 0,
    Lonely = 1,
    Brave = 2,
    Adamant = 3,
    Naughty = 4,
    Bold = 5,
    Docile = 6,
    Relaxed = 7,
    Impish = 8,
    Lax = 9,
    Timid = 10,
    Hasty = 11,
    Serious = 12,
    Jolly = 13,
    Naive = 14,
    Modest = 15,
    Mild = 16,
    Quiet = 17,
    Bashful = 18,
    Rash = 19,
    Calm = 20,
    Gentle = 21,
    Sassy = 22,
    Careful = 23,
    Quirky = 24,
}

impl From<Nature> for Option<(Stat, Stat)> {
    fn from(value: Nature) -> Self {
        use Nature::*;
        use Stat::*;

        match value {
            Hardy => None,
            Lonely => Some((Attack, Defense)),
            Brave => Some((Attack, Speed)),
            Adamant => Some((Attack, SpAtk)),
            Naughty => Some((Attack, SpDef)),
            Bold => Some((Defense, Attack)),
            Docile => None,
            Relaxed => Some((Defense, Speed)),
            Impish => Some((Defense, SpAtk)),
            Lax => Some((Defense, SpDef)),
            Timid => Some((Speed, Attack)),
            Hasty => Some((Speed, Defense)),
            Serious => None,
            Jolly => Some((Speed, SpAtk)),
            Naive => Some((Speed, SpDef)),
            Modest => Some((SpAtk, Attack)),
            Mild => Some((SpAtk, Defense)),
            Quiet => Some((SpAtk, Speed)),
            Bashful => None,
            Rash => Some((SpAtk, SpDef)),
            Calm => Some((SpDef, Attack)),
            Gentle => Some((SpDef, Defense)),
            Sassy => Some((SpDef, Speed)),
            Careful => Some((SpDef, SpAtk)),
            Quirky => None,
        }
    }
}

pub enum Stat {
    HitPoints,
    Attack,
    Defense,
    SpAtk,
    SpDef,
    Speed,
}

pub enum Flavor {
    Spicy,
    Sour,
    Dry,
    Bitter,
    Sweet,
}

impl From<Flavor> for Stat {
    fn from(value: Flavor) -> Self {
        use Flavor::*;
        use Stat::*;

        match value {
            Spicy => Attack,
            Sour => Defense,
            Dry => SpAtk,
            Bitter => SpDef,
            Sweet => Speed,
        }
    }
}

impl From<Stat> for Option<Flavor> {
    fn from(value: Stat) -> Self {
        use Flavor::*;
        use Stat::*;

        match value {
            HitPoints => None,
            Attack => Some(Spicy),
            Defense => Some(Sour),
            SpAtk => Some(Dry),
            SpDef => Some(Bitter),
            Speed => Some(Sweet),
        }
    }
}

pub enum Language {
    Hacked = 0,
    Japanese = 1,
    English = 2,
    French = 3,
    Italian = 4,
    German = 5,
    Unused6 = 6,
    Spanish = 7,
    Korean = 8,
    ChineseS = 9,
    ChineseT = 10,
}

pub enum Gender {
    Male = 0,
    Female = 1,
    Genderless = 2,
}

pub enum BinaryGender {
    Male,
    Female,
}

pub enum StatusCondition {
    Asleep(u8),
    Poisoned,
    Burned,
    Frozen,
    Paralyzed,
    Toxic,
}

#[derive(Debug)]
pub enum Species {
    Bulbasaur,
    Ivysaur,
    Venusaur,
    Charmander,
    Charmeleon,
    Charizard,
    Squirtle,
    Wartortle,
    Blastoise,
    Caterpie,
    Metapod,
    Butterfree,
    Weedle,
    Kakuna,
    Beedrill,
    Pidgey,
    Pidgeotto,
    Pidgeot,
    Rattata,
    Raticate,
    Spearow,
    Fearow,
    Ekans,
    Arbok,
    Pikachu,
    Raichu,
    Sandshrew,
    Sandslash,
    NidoranF,
    Nidorina,
    Nidoqueen,
    NidoranM,
    Nidorino,
    Nidoking,
    Clefairy,
    Clefable,
    Vulpix,
    Ninetales,
    Jigglypuff,
    Wigglytuff,
    Zubat,
    Golbat,
    Oddish,
    Gloom,
    Vileplume,
    Paras,
    Parasect,
    Venonat,
    Venomoth,
    Diglett,
    Dugtrio,
    Meowth,
    Persian,
    Psyduck,
    Golduck,
    Mankey,
    Primeape,
    Growlithe,
    Arcanine,
    Poliwag,
    Poliwhirl,
    Poliwrath,
    Abra,
    Kadabra,
    Alakazam,
    Machop,
    Machoke,
    Machamp,
    Bellsprout,
    Weepinbell,
    Victreebel,
    Tentacool,
    Tentacruel,
    Geodude,
    Graveler,
    Golem,
    Ponyta,
    Rapidash,
    Slowpoke,
    Slowbro,
    Magnemite,
    Magneton,
    Farfetchd,
    Doduo,
    Dodrio,
    Seel,
    Dewgong,
    Grimer,
    Muk,
    Shellder,
    Cloyster,
    Gastly,
    Haunter,
    Gengar,
    Onix,
    Drowzee,
    Hypno,
    Krabby,
    Kingler,
    Voltorb,
    Electrode,
    Exeggcute,
    Exeggutor,
    Cubone,
    Marowak,
    Hitmonlee,
    Hitmonchan,
    Lickitung,
    Koffing,
    Weezing,
    Rhyhorn,
    Rhydon,
    Chansey,
    Tangela,
    Kangaskhan,
    Horsea,
    Seadra,
    Goldeen,
    Seaking,
    Staryu,
    Starmie,
    MrMime,
    Scyther,
    Jynx,
    Electabuzz,
    Magmar,
    Pinsir,
    Tauros,
    Magikarp,
    Gyarados,
    Lapras,
    Ditto,
    Eevee,
    Vaporeon,
    Jolteon,
    Flareon,
    Porygon,
    Omanyte,
    Omastar,
    Kabuto,
    Kabutops,
    Aerodactyl,
    Snorlax,
    Articuno,
    Zapdos,
    Moltres,
    Dratini,
    Dragonair,
    Dragonite,
    Mewtwo,
    Mew,
    Chikorita,
    Bayleef,
    Meganium,
    Cyndaquil,
    Quilava,
    Typhlosion,
    Totodile,
    Croconaw,
    Feraligatr,
    Sentret,
    Furret,
    Hoothoot,
    Noctowl,
    Ledyba,
    Ledian,
    Spinarak,
    Ariados,
    Crobat,
    Chinchou,
    Lanturn,
    Pichu,
    Cleffa,
    Igglybuff,
    Togepi,
    Togetic,
    Natu,
    Xatu,
    Mareep,
    Flaaffy,
    Ampharos,
    Bellossom,
    Marill,
    Azumarill,
    Sudowoodo,
    Politoed,
    Hoppip,
    Skiploom,
    Jumpluff,
    Aipom,
    Sunkern,
    Sunflora,
    Yanma,
    Wooper,
    Quagsire,
    Espeon,
    Umbreon,
    Murkrow,
    Slowking,
    Misdreavus,
    Unown,
    Wobbuffet,
    Girafarig,
    Pineco,
    Forretress,
    Dunsparce,
    Gligar,
    Steelix,
    Snubbull,
    Granbull,
    Qwilfish,
    Scizor,
    Shuckle,
    Heracross,
    Sneasel,
    Teddiursa,
    Ursaring,
    Slugma,
    Magcargo,
    Swinub,
    Piloswine,
    Corsola,
    Remoraid,
    Octillery,
    Delibird,
    Mantine,
    Skarmory,
    Houndour,
    Houndoom,
    Kingdra,
    Phanpy,
    Donphan,
    Porygon2,
    Stantler,
    Smeargle,
    Tyrogue,
    Hitmontop,
    Smoochum,
    Elekid,
    Magby,
    Miltank,
    Blissey,
    Raikou,
    Entei,
    Suicune,
    Larvitar,
    Pupitar,
    Tyranitar,
    Lugia,
    HoOh,
    Celebi,
    Treecko,
    Grovyle,
    Sceptile,
    Torchic,
    Combusken,
    Blaziken,
    Mudkip,
    Marshtomp,
    Swampert,
    Poochyena,
    Mightyena,
    Zigzagoon,
    Linoone,
    Wurmple,
    Silcoon,
    Beautifly,
    Cascoon,
    Dustox,
    Lotad,
    Lombre,
    Ludicolo,
    Seedot,
    Nuzleaf,
    Shiftry,
    Taillow,
    Swellow,
    Wingull,
    Pelipper,
    Ralts,
    Kirlia,
    Gardevoir,
    Surskit,
    Masquerain,
    Shroomish,
    Breloom,
    Slakoth,
    Vigoroth,
    Slaking,
    Nincada,
    Ninjask,
    Shedinja,
    Whismur,
    Loudred,
    Exploud,
    Makuhita,
    Hariyama,
    Azurill,
    Nosepass,
    Skitty,
    Delcatty,
    Sableye,
    Mawile,
    Aron,
    Lairon,
    Aggron,
    Meditite,
    Medicham,
    Electrike,
    Manectric,
    Plusle,
    Minun,
    Volbeat,
    Illumise,
    Roselia,
    Gulpin,
    Swalot,
    Carvanha,
    Sharpedo,
    Wailmer,
    Wailord,
    Numel,
    Camerupt,
    Torkoal,
    Spoink,
    Grumpig,
    Spinda,
    Trapinch,
    Vibrava,
    Flygon,
    Cacnea,
    Cacturne,
    Swablu,
    Altaria,
    Zangoose,
    Seviper,
    Lunatone,
    Solrock,
    Barboach,
    Whiscash,
    Corphish,
    Crawdaunt,
    Baltoy,
    Claydol,
    Lileep,
    Cradily,
    Anorith,
    Armaldo,
    Feebas,
    Milotic,
    Castform,
    Kecleon,
    Shuppet,
    Banette,
    Duskull,
    Dusclops,
    Tropius,
    Chimecho,
    Absol,
    Wynaut,
    Snorunt,
    Glalie,
    Spheal,
    Sealeo,
    Walrein,
    Clamperl,
    Huntail,
    Gorebyss,
    Relicanth,
    Luvdisc,
    Bagon,
    Shelgon,
    Salamence,
    Beldum,
    Metang,
    Metagross,
    Regirock,
    Regice,
    Registeel,
    Latias,
    Latios,
    Kyogre,
    Groudon,
    Rayquaza,
    Jirachi,
    Deoxys,
    Turtwig,
    Grotle,
    Torterra,
    Chimchar,
    Monferno,
    Infernape,
    Piplup,
    Prinplup,
    Empoleon,
    Starly,
    Staravia,
    Staraptor,
    Bidoof,
    Bibarel,
    Kricketot,
    Kricketune,
    Shinx,
    Luxio,
    Luxray,
    Budew,
    Roserade,
    Cranidos,
    Rampardos,
    Shieldon,
    Bastiodon,
    Burmy,
    Wormadam,
    Mothim,
    Combee,
    Vespiquen,
    Pachirisu,
    Buizel,
    Floatzel,
    Cherubi,
    Cherrim,
    Shellos,
    Gastrodon,
    Ambipom,
    Drifloon,
    Drifblim,
    Buneary,
    Lopunny,
    Mismagius,
    Honchkrow,
    Glameow,
    Purugly,
    Chingling,
    Stunky,
    Skuntank,
    Bronzor,
    Bronzong,
    Bonsly,
    MimeJr,
    Happiny,
    Chatot,
    Spiritomb,
    Gible,
    Gabite,
    Garchomp,
    Munchlax,
    Riolu,
    Lucario,
    Hippopotas,
    Hippowdon,
    Skorupi,
    Drapion,
    Croagunk,
    Toxicroak,
    Carnivine,
    Finneon,
    Lumineon,
    Mantyke,
    Snover,
    Abomasnow,
    Weavile,
    Magnezone,
    Lickilicky,
    Rhyperior,
    Tangrowth,
    Electivire,
    Magmortar,
    Togekiss,
    Yanmega,
    Leafeon,
    Glaceon,
    Gliscor,
    Mamoswine,
    PorygonZ,
    Gallade,
    Probopass,
    Dusknoir,
    Froslass,
    Rotom,
    Uxie,
    Mesprit,
    Azelf,
    Dialga,
    Palkia,
    Heatran,
    Regigigas,
    Giratina,
    Cresselia,
    Phione,
    Manaphy,
    Darkrai,
    Shaymin,
    Arceus,
    Victini,
    Snivy,
    Servine,
    Serperior,
    Tepig,
    Pignite,
    Emboar,
    Oshawott,
    Dewott,
    Samurott,
    Patrat,
    Watchog,
    Lillipup,
    Herdier,
    Stoutland,
    Purrloin,
    Liepard,
    Pansage,
    Simisage,
    Pansear,
    Simisear,
    Panpour,
    Simipour,
    Munna,
    Musharna,
    Pidove,
    Tranquill,
    Unfezant,
    Blitzle,
    Zebstrika,
    Roggenrola,
    Boldore,
    Gigalith,
    Woobat,
    Swoobat,
    Drilbur,
    Excadrill,
    Audino,
    Timburr,
    Gurdurr,
    Conkeldurr,
    Tympole,
    Palpitoad,
    Seismitoad,
    Throh,
    Sawk,
    Sewaddle,
    Swadloon,
    Leavanny,
    Venipede,
    Whirlipede,
    Scolipede,
    Cottonee,
    Whimsicott,
    Petilil,
    Lilligant,
    Basculin,
    Sandile,
    Krokorok,
    Krookodile,
    Darumaka,
    Darmanitan,
    Maractus,
    Dwebble,
    Crustle,
    Scraggy,
    Scrafty,
    Sigilyph,
    Yamask,
    Cofagrigus,
    Tirtouga,
    Carracosta,
    Archen,
    Archeops,
    Trubbish,
    Garbodor,
    Zorua,
    Zoroark,
    Minccino,
    Cinccino,
    Gothita,
    Gothorita,
    Gothitelle,
    Solosis,
    Duosion,
    Reuniclus,
    Ducklett,
    Swanna,
    Vanillite,
    Vanillish,
    Vanilluxe,
    Deerling,
    Sawsbuck,
    Emolga,
    Karrablast,
    Escavalier,
    Foongus,
    Amoonguss,
    Frillish,
    Jellicent,
    Alomomola,
    Joltik,
    Galvantula,
    Ferroseed,
    Ferrothorn,
    Klink,
    Klang,
    Klinklang,
    Tynamo,
    Eelektrik,
    Eelektross,
    Elgyem,
    Beheeyem,
    Litwick,
    Lampent,
    Chandelure,
    Axew,
    Fraxure,
    Haxorus,
    Cubchoo,
    Beartic,
    Cryogonal,
    Shelmet,
    Accelgor,
    Stunfisk,
    Mienfoo,
    Mienshao,
    Druddigon,
    Golett,
    Golurk,
    Pawniard,
    Bisharp,
    Bouffalant,
    Rufflet,
    Braviary,
    Vullaby,
    Mandibuzz,
    Heatmor,
    Durant,
    Deino,
    Zweilous,
    Hydreigon,
    Larvesta,
    Volcarona,
    Cobalion,
    Terrakion,
    Virizion,
    Tornadus,
    Thundurus,
    Reshiram,
    Zekrom,
    Landorus,
    Kyurem,
    Keldeo,
    Meloetta,
    Genesect,
    Chespin,
    Quilladin,
    Chesnaught,
    Fennekin,
    Braixen,
    Delphox,
    Froakie,
    Frogadier,
    Greninja,
    Bunnelby,
    Diggersby,
    Fletchling,
    Fletchinder,
    Talonflame,
    Scatterbug,
    Spewpa,
    Vivillon,
    Litleo,
    Pyroar,
    Flabébé,
    Floette,
    Florges,
    Skiddo,
    Gogoat,
    Pancham,
    Pangoro,
    Furfrou,
    Espurr,
    Meowstic,
    Honedge,
    Doublade,
    Aegislash,
    Spritzee,
    Aromatisse,
    Swirlix,
    Slurpuff,
    Inkay,
    Malamar,
    Binacle,
    Barbaracle,
    Skrelp,
    Dragalge,
    Clauncher,
    Clawitzer,
    Helioptile,
    Heliolisk,
    Tyrunt,
    Tyrantrum,
    Amaura,
    Aurorus,
    Sylveon,
    Hawlucha,
    Dedenne,
    Carbink,
    Goomy,
    Sliggoo,
    Goodra,
    Klefki,
    Phantump,
    Trevenant,
    Pumpkaboo,
    Gourgeist,
    Bergmite,
    Avalugg,
    Noibat,
    Noivern,
    Xerneas,
    Yveltal,
    Zygarde,
    Diancie,
    Hoopa,
    Volcanion,
    Rowlet,
    Dartrix,
    Decidueye,
    Litten,
    Torracat,
    Incineroar,
    Popplio,
    Brionne,
    Primarina,
    Pikipek,
    Trumbeak,
    Toucannon,
    Yungoos,
    Gumshoos,
    Grubbin,
    Charjabug,
    Vikavolt,
    Crabrawler,
    Crabominable,
    Oricorio,
    Cutiefly,
    Ribombee,
    Rockruff,
    Lycanroc,
    Wishiwashi,
    Mareanie,
    Toxapex,
    Mudbray,
    Mudsdale,
    Dewpider,
    Araquanid,
    Fomantis,
    Lurantis,
    Morelull,
    Shiinotic,
    Salandit,
    Salazzle,
    Stufful,
    Bewear,
    Bounsweet,
    Steenee,
    Tsareena,
    Comfey,
    Oranguru,
    Passimian,
    Wimpod,
    Golisopod,
    Sandygast,
    Palossand,
    Pyukumuku,
    TypeNull,
    Silvally,
    Minior,
    Komala,
    Turtonator,
    Togedemaru,
    Mimikyu,
    Bruxish,
    Drampa,
    Dhelmise,
    Jangmoo,
    Hakamoo,
    Kommoo,
    TapuKoko,
    TapuLele,
    TapuBulu,
    TapuFini,
    Cosmog,
    Cosmoem,
    Solgaleo,
    Lunala,
    Nihilego,
    Buzzwole,
    Pheromosa,
    Xurkitree,
    Celesteela,
    Kartana,
    Guzzlord,
    Necrozma,
    Magearna,
    Marshadow,
    Poipole,
    Naganadel,
    Stakataka,
    Blacephalon,
    Zeraora,
    Meltan,
    Melmetal,
    Grookey,
    Thwackey,
    Rillaboom,
    Scorbunny,
    Raboot,
    Cinderace,
    Sobble,
    Drizzile,
    Inteleon,
    Skwovet,
    Greedent,
    Rookidee,
    Corvisquire,
    Corviknight,
    Blipbug,
    Dottler,
    Orbeetle,
    Nickit,
    Thievul,
    Gossifleur,
    Eldegoss,
    Wooloo,
    Dubwool,
    Chewtle,
    Drednaw,
    Yamper,
    Boltund,
    Rolycoly,
    Carkol,
    Coalossal,
    Applin,
    Flapple,
    Appletun,
    Silicobra,
    Sandaconda,
    Cramorant,
    Arrokuda,
    Barraskewda,
    Toxel,
    Toxtricity,
    Sizzlipede,
    Centiskorch,
    Clobbopus,
    Grapploct,
    Sinistea,
    Polteageist,
    Hatenna,
    Hattrem,
    Hatterene,
    Impidimp,
    Morgrem,
    Grimmsnarl,
    Obstagoon,
    Perrserker,
    Cursola,
    Sirfetchd,
    MrRime,
    Runerigus,
    Milcery,
    Alcremie,
    Falinks,
    Pincurchin,
    Snom,
    Frosmoth,
    Stonjourner,
    Eiscue,
    Indeedee,
    Morpeko,
    Cufant,
    Copperajah,
    Dracozolt,
    Arctozolt,
    Dracovish,
    Arctovish,
    Duraludon,
    Dreepy,
    Drakloak,
    Dragapult,
    Zacian,
    Zamazenta,
    Eternatus,
    Kubfu,
    Urshifu,
    Zarude,
    Regieleki,
    Regidrago,
    Glastrier,
    Spectrier,
    Calyrex,
    Wyrdeer,
    Kleavor,
    Ursaluna,
    Basculegion,
    Sneasler,
    Overqwil,
    Enamorus,
    Sprigatito,
    Floragato,
    Meowscarada,
    Fuecoco,
    Crocalor,
    Skeledirge,
    Quaxly,
    Quaxwell,
    Quaquaval,
    Lechonk,
    Oinkologne,
    Tarountula,
    Spidops,
    Nymble,
    Lokix,
    Pawmi,
    Pawmo,
    Pawmot,
    Tandemaus,
    Maushold,
    Fidough,
    Dachsbun,
    Smoliv,
    Dolliv,
    Arboliva,
    Squawkabilly,
    Nacli,
    Naclstack,
    Garganacl,
    Charcadet,
    Armarouge,
    Ceruledge,
    Tadbulb,
    Bellibolt,
    Wattrel,
    Kilowattrel,
    Maschiff,
    Mabosstiff,
    Shroodle,
    Grafaiai,
    Bramblin,
    Brambleghast,
    Toedscool,
    Toedscruel,
    Klawf,
    Capsakid,
    Scovillain,
    Rellor,
    Rabsca,
    Flittle,
    Espathra,
    Tinkatink,
    Tinkatuff,
    Tinkaton,
    Wiglett,
    Wugtrio,
    Bombirdier,
    Finizen,
    Palafin,
    Varoom,
    Revavroom,
    Cyclizar,
    Orthworm,
    Glimmet,
    Glimmora,
    Greavard,
    Houndstone,
    Flamigo,
    Cetoddle,
    Cetitan,
    Veluza,
    Dondozo,
    Tatsugiri,
    Annihilape,
    Clodsire,
    Farigiraf,
    Dudunsparce,
    Kingambit,
    GreatTusk,
    ScreamTail,
    BruteBonnet,
    FlutterMane,
    SlitherWing,
    SandyShocks,
    IronTreads,
    IronBundle,
    IronHands,
    IronJugulis,
    IronMoth,
    IronThorns,
    Frigibax,
    Arctibax,
    Baxcalibur,
    Gimmighoul,
    Gholdengo,
    WoChien,
    ChienPao,
    TingLu,
    ChiYu,
    RoaringMoon,
    IronValiant,
    Koraidon,
    Miraidon,
    WalkingWake,
    IronLeaves,
    Dipplin,
    Poltchageist,
    Sinistcha,
    Okidogi,
    Munkidori,
    Fezandipiti,
    Ogerpon,
}

#[derive(Debug)]
pub enum Ability {
    Stench,
    Drizzle,
    SpeedBoost,
    BattleArmor,
    Sturdy,
    Damp,
    Limber,
    SandVeil,
    Static,
    VoltAbsorb,
    WaterAbsorb,
    Oblivious,
    CloudNine,
    CompoundEyes,
    Insomnia,
    ColorChange,
    Immunity,
    FlashFire,
    ShieldDust,
    OwnTempo,
    SuctionCups,
    Intimidate,
    ShadowTag,
    RoughSkin,
    WonderGuard,
    Levitate,
    EffectSpore,
    Synchronize,
    ClearBody,
    NaturalCure,
    LightningRod,
    SereneGrace,
    SwiftSwim,
    Chlorophyll,
    Illuminate,
    Trace,
    HugePower,
    PoisonPoint,
    InnerFocus,
    MagmaArmor,
    WaterVeil,
    MagnetPull,
    Soundproof,
    RainDish,
    SandStream,
    Pressure,
    ThickFat,
    EarlyBird,
    FlameBody,
    RunAway,
    KeenEye,
    HyperCutter,
    Pickup,
    Truant,
    Hustle,
    CuteCharm,
    Plus,
    Minus,
    Forecast,
    StickyHold,
    ShedSkin,
    Guts,
    MarvelScale,
    LiquidOoze,
    Overgrow,
    Blaze,
    Torrent,
    Swarm,
    RockHead,
    Drought,
    ArenaTrap,
    VitalSpirit,
    WhiteSmoke,
    PurePower,
    ShellArmor,
    AirLock,
    TangledFeet,
    MotorDrive,
    Rivalry,
    Steadfast,
    SnowCloak,
    Gluttony,
    AngerPoint,
    Unburden,
    Heatproof,
    Simple,
    DrySkin,
    Download,
    IronFist,
    PoisonHeal,
    Adaptability,
    SkillLink,
    Hydration,
    SolarPower,
    QuickFeet,
    Normalize,
    Sniper,
    MagicGuard,
    NoGuard,
    Stall,
    Technician,
    LeafGuard,
    Klutz,
    MoldBreaker,
    SuperLuck,
    Aftermath,
    Anticipation,
    Forewarn,
    Unaware,
    TintedLens,
    Filter,
    SlowStart,
    Scrappy,
    StormDrain,
    IceBody,
    SolidRock,
    SnowWarning,
    HoneyGather,
    Frisk,
    Reckless,
    Multitype,
    FlowerGift,
    BadDreams,
    Pickpocket,
    SheerForce,
    Contrary,
    Unnerve,
    Defiant,
    Defeatist,
    CursedBody,
    Healer,
    FriendGuard,
    WeakArmor,
    HeavyMetal,
    LightMetal,
    Multiscale,
    ToxicBoost,
    FlareBoost,
    Harvest,
    Telepathy,
    Moody,
    Overcoat,
    PoisonTouch,
    Regenerator,
    BigPecks,
    SandRush,
    WonderSkin,
    Analytic,
    Illusion,
    Imposter,
    Infiltrator,
    Mummy,
    Moxie,
    Justified,
    Rattled,
    MagicBounce,
    SapSipper,
    Prankster,
    SandForce,
    IronBarbs,
    ZenMode,
    VictoryStar,
    Turboblaze,
    Teravolt,
    AromaVeil,
    FlowerVeil,
    CheekPouch,
    Protean,
    FurCoat,
    Magician,
    Bulletproof,
    Competitive,
    StrongJaw,
    Refrigerate,
    SweetVeil,
    StanceChange,
    GaleWings,
    MegaLauncher,
    GrassPelt,
    Symbiosis,
    ToughClaws,
    Pixilate,
    Gooey,
    Aerilate,
    ParentalBond,
    DarkAura,
    FairyAura,
    AuraBreak,
    PrimordialSea,
    DesolateLand,
    DeltaStream,
    Stamina,
    WimpOut,
    EmergencyExit,
    WaterCompaction,
    Merciless,
    ShieldsDown,
    Stakeout,
    WaterBubble,
    Steelworker,
    Berserk,
    SlushRush,
    LongReach,
    LiquidVoice,
    Triage,
    Galvanize,
    SurgeSurfer,
    Schooling,
    Disguise,
    BattleBond,
    PowerConstruct,
    Corrosion,
    Comatose,
    QueenlyMajesty,
    InnardsOut,
    Dancer,
    Battery,
    Fluffy,
    Dazzling,
    SoulHeart,
    TanglingHair,
    Receiver,
    PowerofAlchemy,
    BeastBoost,
    RKSSystem,
    ElectricSurge,
    PsychicSurge,
    MistySurge,
    GrassySurge,
    FullMetalBody,
    ShadowShield,
    PrismArmor,
    Neuroforce,
    IntrepidSword,
    DauntlessShield,
    Libero,
    BallFetch,
    CottonDown,
    PropellerTail,
    MirrorArmor,
    GulpMissile,
    Stalwart,
    SteamEngine,
    PunkRock,
    SandSpit,
    IceScales,
    Ripen,
    IceFace,
    PowerSpot,
    Mimicry,
    ScreenCleaner,
    SteelySpirit,
    PerishBody,
    WanderingSpirit,
    GorillaTactics,
    NeutralizingGas,
    PastelVeil,
    HungerSwitch,
    QuickDraw,
    UnseenFist,
    CuriousMedicine,
    Transistor,
    DragonsMaw,
    ChillingNeigh,
    GrimNeigh,
    AsOneI,
    AsOneG,
    LingeringAroma,
    SeedSower,
    ThermalExchange,
    AngerShell,
    PurifyingSalt,
    WellBakedBody,
    WindRider,
    GuardDog,
    RockyPayload,
    WindPower,
    ZerotoHero,
    Commander,
    Electromorphosis,
    Protosynthesis,
    QuarkDrive,
    GoodasGold,
    VesselofRuin,
    SwordofRuin,
    TabletsofRuin,
    BeadsofRuin,
    OrichalcumPulse,
    HadronEngine,
    Opportunist,
    CudChew,
    Sharpness,
    SupremeOverlord,
    Costar,
    ToxicDebris,
    ArmorTail,
    EarthEater,
    MyceliumMight,
    Hospitality,
    MindsEye,
    EmbodyAspect0,
    EmbodyAspect1,
    EmbodyAspect2,
    EmbodyAspect3,
    ToxicChain,
    SupersweetSyrup,
}

pub enum PokeBall {
    Master,
    Ultra,
    Great,
    Standard,
    Safari,
    Net,
    Dive,
    Nest,
    Repeat,
    Timer,
    Luxury,
    Premier,
    Dusk,
    Heal,
    Quick,
    Cherish,
    Fast,
    Level,
    Lure,
    Heavy,
    Love,
    Friend,
    Moon,
    Sport,
    Dream,
    Beast,
    Park,
    Strange,
    HisuiStandard,
    HisuiGreat,
    HisuiUltra,
    Feather,
    Wing,
    Jet,
    HisuiHeavy,
    Leaden,
    Gigaton,
    Origin,
}

pub enum Region3DS {
    None = 0,
    Japan = 1,
    NorthAmerica = 2,
    Europe = 3,
    China = 4,
    Korea = 5,
    Taiwan = 6,
}

pub enum Move {
    Pound,
    KarateChop,
    DoubleSlap,
    CometPunch,
    MegaPunch,
    PayDay,
    FirePunch,
    IcePunch,
    ThunderPunch,
    Scratch,
    ViseGrip,
    Guillotine,
    RazorWind,
    SwordsDance,
    Cut,
    Gust,
    WingAttack,
    Whirlwind,
    Fly,
    Bind,
    Slam,
    VineWhip,
    Stomp,
    DoubleKick,
    MegaKick,
    JumpKick,
    RollingKick,
    SandAttack,
    Headbutt,
    HornAttack,
    FuryAttack,
    HornDrill,
    Tackle,
    BodySlam,
    Wrap,
    TakeDown,
    Thrash,
    DoubleEdge,
    TailWhip,
    PoisonSting,
    Twineedle,
    PinMissile,
    Leer,
    Bite,
    Growl,
    Roar,
    Sing,
    Supersonic,
    SonicBoom,
    Disable,
    Acid,
    Ember,
    Flamethrower,
    Mist,
    WaterGun,
    HydroPump,
    Surf,
    IceBeam,
    Blizzard,
    Psybeam,
    BubbleBeam,
    AuroraBeam,
    HyperBeam,
    Peck,
    DrillPeck,
    Submission,
    LowKick,
    Counter,
    SeismicToss,
    Strength,
    Absorb,
    MegaDrain,
    LeechSeed,
    Growth,
    RazorLeaf,
    SolarBeam,
    PoisonPowder,
    StunSpore,
    SleepPowder,
    PetalDance,
    StringShot,
    DragonRage,
    FireSpin,
    ThunderShock,
    Thunderbolt,
    ThunderWave,
    Thunder,
    RockThrow,
    Earthquake,
    Fissure,
    Dig,
    Toxic,
    Confusion,
    Psychic,
    Hypnosis,
    Meditate,
    Agility,
    QuickAttack,
    Rage,
    Teleport,
    NightShade,
    Mimic,
    Screech,
    DoubleTeam,
    Recover,
    Harden,
    Minimize,
    Smokescreen,
    ConfuseRay,
    Withdraw,
    DefenseCurl,
    Barrier,
    LightScreen,
    Haze,
    Reflect,
    FocusEnergy,
    Bide,
    Metronome,
    MirrorMove,
    SelfDestruct,
    EggBomb,
    Lick,
    Smog,
    Sludge,
    BoneClub,
    FireBlast,
    Waterfall,
    Clamp,
    Swift,
    SkullBash,
    SpikeCannon,
    Constrict,
    Amnesia,
    Kinesis,
    SoftBoiled,
    HighJumpKick,
    Glare,
    DreamEater,
    PoisonGas,
    Barrage,
    LeechLife,
    LovelyKiss,
    SkyAttack,
    Transform,
    Bubble,
    DizzyPunch,
    Spore,
    Flash,
    Psywave,
    Splash,
    AcidArmor,
    Crabhammer,
    Explosion,
    FurySwipes,
    Bonemerang,
    Rest,
    RockSlide,
    HyperFang,
    Sharpen,
    Conversion,
    TriAttack,
    SuperFang,
    Slash,
    Substitute,
    Struggle,
    Sketch,
    TripleKick,
    Thief,
    SpiderWeb,
    MindReader,
    Nightmare,
    FlameWheel,
    Snore,
    Curse,
    Flail,
    Conversion2,
    Aeroblast,
    CottonSpore,
    Reversal,
    Spite,
    PowderSnow,
    Protect,
    MachPunch,
    ScaryFace,
    FeintAttack,
    SweetKiss,
    BellyDrum,
    SludgeBomb,
    MudSlap,
    Octazooka,
    Spikes,
    ZapCannon,
    Foresight,
    DestinyBond,
    PerishSong,
    IcyWind,
    Detect,
    BoneRush,
    LockOn,
    Outrage,
    Sandstorm,
    GigaDrain,
    Endure,
    Charm,
    Rollout,
    FalseSwipe,
    Swagger,
    MilkDrink,
    Spark,
    FuryCutter,
    SteelWing,
    MeanLook,
    Attract,
    SleepTalk,
    HealBell,
    Return,
    Present,
    Frustration,
    Safeguard,
    PainSplit,
    SacredFire,
    Magnitude,
    DynamicPunch,
    Megahorn,
    DragonBreath,
    BatonPass,
    Encore,
    Pursuit,
    RapidSpin,
    SweetScent,
    IronTail,
    MetalClaw,
    VitalThrow,
    MorningSun,
    Synthesis,
    Moonlight,
    HiddenPower,
    CrossChop,
    Twister,
    RainDance,
    SunnyDay,
    Crunch,
    MirrorCoat,
    PsychUp,
    ExtremeSpeed,
    AncientPower,
    ShadowBall,
    FutureSight,
    RockSmash,
    Whirlpool,
    BeatUp,
    FakeOut,
    Uproar,
    Stockpile,
    SpitUp,
    Swallow,
    HeatWave,
    Hail,
    Torment,
    Flatter,
    WillOWisp,
    Memento,
    Facade,
    FocusPunch,
    SmellingSalts,
    FollowMe,
    NaturePower,
    Charge,
    Taunt,
    HelpingHand,
    Trick,
    RolePlay,
    Wish,
    Assist,
    Ingrain,
    Superpower,
    MagicCoat,
    Recycle,
    Revenge,
    BrickBreak,
    Yawn,
    KnockOff,
    Endeavor,
    Eruption,
    SkillSwap,
    Imprison,
    Refresh,
    Grudge,
    Snatch,
    SecretPower,
    Dive,
    ArmThrust,
    Camouflage,
    TailGlow,
    LusterPurge,
    MistBall,
    FeatherDance,
    TeeterDance,
    BlazeKick,
    MudSport,
    IceBall,
    NeedleArm,
    SlackOff,
    HyperVoice,
    PoisonFang,
    CrushClaw,
    BlastBurn,
    HydroCannon,
    MeteorMash,
    Astonish,
    WeatherBall,
    Aromatherapy,
    FakeTears,
    AirCutter,
    Overheat,
    OdorSleuth,
    RockTomb,
    SilverWind,
    MetalSound,
    GrassWhistle,
    Tickle,
    CosmicPower,
    WaterSpout,
    SignalBeam,
    ShadowPunch,
    Extrasensory,
    SkyUppercut,
    SandTomb,
    SheerCold,
    MuddyWater,
    BulletSeed,
    AerialAce,
    IcicleSpear,
    IronDefense,
    Block,
    Howl,
    DragonClaw,
    FrenzyPlant,
    BulkUp,
    Bounce,
    MudShot,
    PoisonTail,
    Covet,
    VoltTackle,
    MagicalLeaf,
    WaterSport,
    CalmMind,
    LeafBlade,
    DragonDance,
    RockBlast,
    ShockWave,
    WaterPulse,
    DoomDesire,
    PsychoBoost,
    Roost,
    Gravity,
    MiracleEye,
    WakeUpSlap,
    HammerArm,
    GyroBall,
    HealingWish,
    Brine,
    NaturalGift,
    Feint,
    Pluck,
    Tailwind,
    Acupressure,
    MetalBurst,
    Uturn,
    CloseCombat,
    Payback,
    Assurance,
    Embargo,
    Fling,
    PsychoShift,
    TrumpCard,
    HealBlock,
    WringOut,
    PowerTrick,
    GastroAcid,
    LuckyChant,
    MeFirst,
    Copycat,
    PowerSwap,
    GuardSwap,
    Punishment,
    LastResort,
    WorrySeed,
    SuckerPunch,
    ToxicSpikes,
    HeartSwap,
    AquaRing,
    MagnetRise,
    FlareBlitz,
    ForcePalm,
    AuraSphere,
    RockPolish,
    PoisonJab,
    DarkPulse,
    NightSlash,
    AquaTail,
    SeedBomb,
    AirSlash,
    XScissor,
    BugBuzz,
    DragonPulse,
    DragonRush,
    PowerGem,
    DrainPunch,
    VacuumWave,
    FocusBlast,
    EnergyBall,
    BraveBird,
    EarthPower,
    Switcheroo,
    GigaImpact,
    NastyPlot,
    BulletPunch,
    Avalanche,
    IceShard,
    ShadowClaw,
    ThunderFang,
    IceFang,
    FireFang,
    ShadowSneak,
    MudBomb,
    PsychoCut,
    ZenHeadbutt,
    MirrorShot,
    FlashCannon,
    RockClimb,
    Defog,
    TrickRoom,
    DracoMeteor,
    Discharge,
    LavaPlume,
    LeafStorm,
    PowerWhip,
    RockWrecker,
    CrossPoison,
    GunkShot,
    IronHead,
    MagnetBomb,
    StoneEdge,
    Captivate,
    StealthRock,
    GrassKnot,
    Chatter,
    Judgment,
    BugBite,
    ChargeBeam,
    WoodHammer,
    AquaJet,
    AttackOrder,
    DefendOrder,
    HealOrder,
    HeadSmash,
    DoubleHit,
    RoarofTime,
    SpacialRend,
    LunarDance,
    CrushGrip,
    MagmaStorm,
    DarkVoid,
    SeedFlare,
    OminousWind,
    ShadowForce,
    HoneClaws,
    WideGuard,
    GuardSplit,
    PowerSplit,
    WonderRoom,
    Psyshock,
    Venoshock,
    Autotomize,
    RagePowder,
    Telekinesis,
    MagicRoom,
    SmackDown,
    StormThrow,
    FlameBurst,
    SludgeWave,
    QuiverDance,
    HeavySlam,
    Synchronoise,
    ElectroBall,
    Soak,
    FlameCharge,
    Coil,
    LowSweep,
    AcidSpray,
    FoulPlay,
    SimpleBeam,
    Entrainment,
    AfterYou,
    Round,
    EchoedVoice,
    ChipAway,
    ClearSmog,
    StoredPower,
    QuickGuard,
    AllySwitch,
    Scald,
    ShellSmash,
    HealPulse,
    Hex,
    SkyDrop,
    ShiftGear,
    CircleThrow,
    Incinerate,
    Quash,
    Acrobatics,
    ReflectType,
    Retaliate,
    FinalGambit,
    Bestow,
    Inferno,
    WaterPledge,
    FirePledge,
    GrassPledge,
    VoltSwitch,
    StruggleBug,
    Bulldoze,
    FrostBreath,
    DragonTail,
    WorkUp,
    Electroweb,
    WildCharge,
    DrillRun,
    DualChop,
    HeartStamp,
    HornLeech,
    SacredSword,
    RazorShell,
    HeatCrash,
    LeafTornado,
    Steamroller,
    CottonGuard,
    NightDaze,
    Psystrike,
    TailSlap,
    Hurricane,
    HeadCharge,
    GearGrind,
    SearingShot,
    TechnoBlast,
    RelicSong,
    SecretSword,
    Glaciate,
    BoltStrike,
    BlueFlare,
    FieryDance,
    FreezeShock,
    IceBurn,
    Snarl,
    IcicleCrash,
    Vcreate,
    FusionFlare,
    FusionBolt,
    FlyingPress,
    MatBlock,
    Belch,
    Rototiller,
    StickyWeb,
    FellStinger,
    PhantomForce,
    TrickorTreat,
    NobleRoar,
    IonDeluge,
    ParabolicCharge,
    ForestsCurse,
    PetalBlizzard,
    FreezeDry,
    DisarmingVoice,
    PartingShot,
    TopsyTurvy,
    DrainingKiss,
    CraftyShield,
    FlowerShield,
    GrassyTerrain,
    MistyTerrain,
    Electrify,
    PlayRough,
    FairyWind,
    Moonblast,
    Boomburst,
    FairyLock,
    KingsShield,
    PlayNice,
    Confide,
    DiamondStorm,
    SteamEruption,
    HyperspaceHole,
    WaterShuriken,
    MysticalFire,
    SpikyShield,
    AromaticMist,
    EerieImpulse,
    VenomDrench,
    Powder,
    Geomancy,
    MagneticFlux,
    HappyHour,
    ElectricTerrain,
    DazzlingGleam,
    Celebrate,
    HoldHands,
    BabyDollEyes,
    Nuzzle,
    HoldBack,
    Infestation,
    PowerUpPunch,
    OblivionWing,
    ThousandArrows,
    ThousandWaves,
    LandsWrath,
    LightofRuin,
    OriginPulse,
    PrecipiceBlades,
    DragonAscent,
    HyperspaceFury,
    BreakneckBlitzP,
    BreakneckBlitzS,
    AllOutPummelingP,
    AllOutPummelingS,
    SupersonicSkystrikeP,
    SupersonicSkystrikeS,
    AcidDownpourP,
    AcidDownpourS,
    TectonicRageP,
    TectonicRageS,
    ContinentalCrushP,
    ContinentalCrushS,
    SavageSpinOutP,
    SavageSpinOutS,
    NeverEndingNightmareP,
    NeverEndingNightmareS,
    CorkscrewCrashP,
    CorkscrewCrashS,
    InfernoOverdriveP,
    InfernoOverdriveS,
    HydroVortexP,
    HydroVortexS,
    BloomDoomP,
    BloomDoomS,
    GigavoltHavocP,
    GigavoltHavocS,
    ShatteredPsycheP,
    ShatteredPsycheS,
    SubzeroSlammerP,
    SubzeroSlammerS,
    DevastatingDrakeP,
    DevastatingDrakeS,
    BlackHoleEclipseP,
    BlackHoleEclipseS,
    TwinkleTackleP,
    TwinkleTackleS,
    Catastropika,
    ShoreUp,
    FirstImpression,
    BanefulBunker,
    SpiritShackle,
    DarkestLariat,
    SparklingAria,
    IceHammer,
    FloralHealing,
    HighHorsepower,
    StrengthSap,
    SolarBlade,
    Leafage,
    Spotlight,
    ToxicThread,
    LaserFocus,
    GearUp,
    ThroatChop,
    PollenPuff,
    AnchorShot,
    PsychicTerrain,
    Lunge,
    FireLash,
    PowerTrip,
    BurnUp,
    SpeedSwap,
    SmartStrike,
    Purify,
    RevelationDance,
    CoreEnforcer,
    TropKick,
    Instruct,
    BeakBlast,
    ClangingScales,
    DragonHammer,
    BrutalSwing,
    AuroraVeil,
    SinisterArrowRaid,
    MaliciousMoonsault,
    OceanicOperetta,
    GuardianofAlola,
    SoulStealing7StarStrike,
    StokedSparksurfer,
    PulverizingPancake,
    ExtremeEvoboost,
    GenesisSupernova,
    ShellTrap,
    FleurCannon,
    PsychicFangs,
    StompingTantrum,
    ShadowBone,
    Accelerock,
    Liquidation,
    PrismaticLaser,
    SpectralThief,
    SunsteelStrike,
    MoongeistBeam,
    TearfulLook,
    ZingZap,
    NaturesMadness,
    MultiAttack,
    TenMVoltThunderbolt,
    MindBlown,
    PlasmaFists,
    PhotonGeyser,
    LightThatBurnstheSky,
    SearingSunrazeSmash,
    MenacingMoonrazeMaelstrom,
    LetsSnuggleForever,
    SplinteredStormshards,
    ClangorousSoulblaze,
    ZippyZap,
    SplishySplash,
    FloatyFall,
    PikaPapow,
    BouncyBubble,
    BuzzyBuzz,
    SizzlySlide,
    GlitzyGlow,
    BaddyBad,
    SappySeed,
    FreezyFrost,
    SparklySwirl,
    VeeveeVolley,
    DoubleIronBash,
    MaxGuard,
    DynamaxCannon,
    SnipeShot,
    JawLock,
    StuffCheeks,
    NoRetreat,
    TarShot,
    MagicPowder,
    DragonDarts,
    Teatime,
    Octolock,
    BoltBeak,
    FishiousRend,
    CourtChange,
    MaxFlare,
    MaxFlutterby,
    MaxLightning,
    MaxStrike,
    MaxKnuckle,
    MaxPhantasm,
    MaxHailstorm,
    MaxOoze,
    MaxGeyser,
    MaxAirstream,
    MaxStarfall,
    MaxWyrmwind,
    MaxMindstorm,
    MaxRockfall,
    MaxQuake,
    MaxDarkness,
    MaxOvergrowth,
    MaxSteelspike,
    ClangorousSoul,
    BodyPress,
    Decorate,
    DrumBeating,
    SnapTrap,
    PyroBall,
    BehemothBlade,
    BehemothBash,
    AuraWheel,
    BreakingSwipe,
    BranchPoke,
    Overdrive,
    AppleAcid,
    GravApple,
    SpiritBreak,
    StrangeSteam,
    LifeDew,
    Obstruct,
    FalseSurrender,
    MeteorAssault,
    Eternabeam,
    SteelBeam,
    ExpandingForce,
    SteelRoller,
    ScaleShot,
    MeteorBeam,
    ShellSideArm,
    MistyExplosion,
    GrassyGlide,
    RisingVoltage,
    TerrainPulse,
    SkitterSmack,
    BurningJealousy,
    LashOut,
    Poltergeist,
    CorrosiveGas,
    Coaching,
    FlipTurn,
    TripleAxel,
    DualWingbeat,
    ScorchingSands,
    JungleHealing,
    WickedBlow,
    SurgingStrikes,
    ThunderCage,
    DragonEnergy,
    FreezingGlare,
    FieryWrath,
    ThunderousKick,
    GlacialLance,
    AstralBarrage,
    EerieSpell,
    DireClaw,
    PsyshieldBash,
    PowerShift,
    StoneAxe,
    SpringtideStorm,
    MysticalPower,
    RagingFury,
    WaveCrash,
    Chloroblast,
    MountainGale,
    VictoryDance,
    HeadlongRush,
    BarbBarrage,
    EsperWing,
    BitterMalice,
    Shelter,
    TripleArrows,
    InfernalParade,
    CeaselessEdge,
    BleakwindStorm,
    WildboltStorm,
    SandsearStorm,
    LunarBlessing,
    TakeHeart,
    TeraBlast,
    SilkTrap,
    AxeKick,
    LastRespects,
    LuminaCrash,
    OrderUp,
    JetPunch,
    SpicyExtract,
    SpinOut,
    PopulationBomb,
    IceSpinner,
    GlaiveRush,
    RevivalBlessing,
    SaltCure,
    TripleDive,
    MortalSpin,
    Doodle,
    FilletAway,
    KowtowCleave,
    FlowerTrick,
    TorchSong,
    AquaStep,
    RagingBull,
    MakeItRain,
    Psyblade,
    HydroSteam,
    Ruination,
    CollisionCourse,
    ElectroDrift,
    ShedTail,
    ChillyReception,
    TidyUp,
    Snowscape,
    Pounce,
    Trailblaze,
    ChillingWater,
    HyperDrill,
    TwinBeam,
    RageFist,
    ArmorCannon,
    BitterBlade,
    DoubleShock,
    GigatonHammer,
    Comeuppance,
    AquaCutter,
    BlazingTorque,
    WickedTorque,
    NoxiousTorque,
    CombatTorque,
    MagicalTorque,
    BloodMoon,
    MatchaGotcha,
    SyrupBomb,
    IvyCudgel,
}

pub enum Type {
    Normal,
    Fighting,
    Flying,
    Poison,
    Ground,
    Rock,
    Bug,
    Ghost,
    Steel,
    Fire,
    Water,
    Grass,
    Electric,
    Psychic,
    Ice,
    Dragon,
    Dark,
    Fairy,
}

pub enum GCRegion {
    NoRegion = 0,
    NTSCJ = 1,
    NTSCU = 2,
    PAL = 3,
}

pub enum GCVersion {
    None = 0,
    FR = 1,
    LG = 2,
    S = 8,
    R = 9,
    E = 10,
    CXD = 11,
}

#[repr(u8)]
pub enum GameVersion {
    Any = 0,
    Unknown = 0xFE,
    Invalid = 0xFF,

    // gen 3
    Sapphire = 1,
    Ruby = 2,
    Emerald = 3,
    FireRed = 4,
    LeafGreen = 5,
    ColosseumXD = 15,

    // gen 4
    Diamond = 10,
    Pearl = 11,
    Platinum = 12,
    HeartGold = 7,
    SoulSilver = 8,

    // gen 5
    White = 20,
    Black = 21,
    White2 = 22,
    Black2 = 23,

    // gen 6
    X = 24,
    Y = 25,
    AlphaSapphire = 26,
    OmegaRuby = 27,

    // gen 7
    Sun = 30,
    Moon = 31,
    UltraSun = 32,
    UltraMoon = 33,

    GO = 34,

    // 3ds vc gen1
    Red = 35,
    Green = 36,
    Blue = 37,
    Yellow = 38,

    // 3ds vc gen2
    Gold = 39,
    Silver = 40,
    Crystal = 41,

    // switch
    LetsGoPikachu = 42,
    LetsGoEevee = 43,
    Sword = 44,
    Shield = 45,
    HOME = 46,
    LegendsArceus = 47,
    BrilliantDiamond = 48,
    ShiningPearl = 49,
    Scarlet = 50,
    Violet = 51,
}

pub struct LearnedMove {
    pub r#move: Move,
    pub max_pp: u8,
    pub current_pp: u8,
    pub pp_up: u8,
}

pub struct MoveSet(LearnedMove, LearnedMove, LearnedMove, LearnedMove);

#[derive(Debug, Clone)]
pub struct Trainer {
    pub name: String,
    pub id: TrainerId,
}

#[derive(Debug, Clone)]
pub enum TrainerId {
    /// used in gens 1-2. no secret id
    SixteenBitSingle(u16),
    /// gens 3-6
    SixteenBit(u16, u16),
    /// gens 7+
    SixDigit(u32),
}

impl Display for TrainerId {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "{:06}",
            match self {
                TrainerId::SixteenBitSingle(v) => *v as u32,
                TrainerId::SixteenBit(v, _) => *v as u32,
                TrainerId::SixDigit(v) => v % 100_000,
            }
        )
    }
}

pub struct StatSpread<K> {
    // Kontainer
    pub hp: K,
    pub attack: K,
    pub defense: K,
    pub speed: K,
    pub sp_attack: K,
    pub sp_def: K,
}

pub type EVs = StatSpread<u8>;

pub struct IV(u8);

impl TryFrom<u8> for IV {
    type Error = ();

    fn try_from(value: u8) -> Result<Self, Self::Error> {
        if value > 31 {
            return Err(());
        }

        Ok(Self(value))
    }
}

impl From<IV> for u8 {
    fn from(value: IV) -> Self {
        value.0
    }
}

pub type IVs = StatSpread<IV>;
