use crate::{
    general::{BinaryGender, Trainer, TrainerId},
    pokemon::PKM,
};

/// https://projectpokemon.org/home/docs/gen-5/bw-save-structure-r60/
pub struct PK5 {
    personality: u32,
    checksum: u16,
    pokedex_id: u16,

    // block A
    held_item: u16,
    ot_id: u16,
    ot_secret_id: u16,
    experience: u32,
    friendship: u8,
    ability: u8,
    markings: u8,
    original_language: u8,
    ev_hp: u8,
    ev_attack: u8,
    ev_defense: u8,
    ev_speed: u8,
    ev_sp_attack: u8,
    ev_sp_defense: u8,
    contest_value_cool: u8,
    contest_value_beauty: u8,
    contest_value_cute: u8,
    contest_value_smart: u8,
    contest_value_tough: u8,
    contest_value_sheen: u8,
    ribbon_set_sinnoh_1: u16,
    ribbon_set_unova: u16,

    // block B
    move_1_id: u16,
    move_2_id: u16,
    move_3_id: u16,
    move_4_id: u16,
    move_1_current_pp: u8,
    move_2_current_pp: u8,
    move_3_current_pp: u8,
    move_4_current_pp: u8,
    move_pp_ups: u32,
    ivs_isegg_isnicknamed: u32,
    ribbon_set_hoenn_1: u16,
    ribbon_set_hoenn_2: u16,
    fateful_gender_altform: u8,
    nature: u8,
    dreamworld_ability_n_flags: u8,
    unknown_1: u32,

    // block C
    nickname: String,
    unknown_2: u8,
    origin_game: u8,
    ribbon_set_sinnoh_3: u16,
    ribbon_set_sinnoh_4: u16,

    // block D
    ot_name: String,
    date_egg_received: [u8; 3],
    date_met: [u8; 3],
    egg_location_dp: u16,
    met_at_location_dp: u16,
    pokerus: u8,
    pokeball: u8,
    met_at_level_ot_gender: u8,
    encounter_type: u8,

    // battle stats
    status: u8,
    unknown_flags: u8, // max 0xF0
    unknown_3: u16,
    level: u8,
    capsule_index: u8,
    current_hp: u16,
    max_hp: u16,
    attack: u16,
    defense: u16,
    speed: u16,
    special_attack: u16,
    special_defense: u16,
    mail_message: Mail5,
}

impl PKM for PK5 {
    fn ot(&self) -> &Trainer {
        &Trainer {
            name: self.ot_name,
            id: TrainerId::SixteenBit(self.ot_id, self.ot_secret_id),
        }
    }

    fn gain_xp(&mut self, xp: u32) -> Vec<super::Progression> {
        self.experience += xp;
        //
        todo!()
    }

    fn defeated(&mut self, other: crate::general::Species) {
        todo!()
    }

    fn use_item(&mut self, item: Item) -> Result<(), ItemUseError> {
        todo!()
    }

    fn damage(&mut self, hp: u16) {
        todo!()
    }

    fn apply_status(&mut self, status: crate::general::StatusCondition) {
        todo!()
    }

    fn species(&self) -> crate::general::Species {
        todo!()
    }

    fn ivs(&self) -> crate::general::IVs {
        todo!()
    }

    fn evs(&self) -> crate::general::EVs {
        todo!()
    }

    fn is_shiny(&self) -> bool {
        todo!()
    }

    fn nickname(&self) -> Option<String> {
        todo!()
    }

    fn held_item(&self) -> Option<Item> {
        todo!()
    }

    fn current_hp(&self) -> u16 {
        todo!()
    }

    fn max_hp(&self) -> u16 {
        todo!()
    }

    fn effective_base_stats(&self) -> crate::general::StatSpread<u16> {
        todo!()
    }

    fn move_set(&self) -> crate::general::MoveSet {
        todo!()
    }
}

pub struct Mail5 {
    author_tid: u16,
    author_sid: u16,
    author_gender: BinaryGender,
    author_language: u8,
    author_version: u8,
    mail_type: u8,
    author_name: String,
    message_ending: u16,
    appear_pkm: u16,
    message_data_1: u16,
    message_data_2: u16,
    message_data_3: u16,
}
